#include <linux/module.h>
#include <linux/init.h>
#include <linux/kernel.h>
#include <linux/fs.h>
#include <linux/cdev.h>
#include <asm/uaccess.h>
#include <linux/sched.h>
#include <linux/wait.h>
#include <linux/atomic.h>

#include "../include/util.h"

MODULE_DESCRIPTION("Use ioctl for notify and wait");
MODULE_AUTHOR("SO2");
MODULE_LICENSE("GPL");

#define LOG_LEVEL	KERN_ALERT

#define MY_MAJOR		42
#define MY_MINOR		0
#define NUM_MINORS		1
#define MODULE_NAME		"print"
#define IOCTL_MESSAGE		"Hello ioctl"

struct device_data_struct {
	struct cdev cdev;
	wait_queue_head_t wq;
	atomic_t flag;
} devs[NUM_MINORS];

static int print_open(struct inode *inode, struct file *file)
{
	struct device_data_struct *data =
		container_of(inode->i_cdev, struct device_data_struct, cdev);

	file->private_data = data;

	return 0;
}

static int print_release(struct inode *inode, struct file *file)
{
	return 0;
}

static long print_ioctl(struct file *file, unsigned int cmd, unsigned long arg)
{
	long ret = 0;

	struct device_data_struct *data = file->private_data;

	switch (cmd) {
	case MY_IOCTL_WAIT:
		printk(LOG_LEVEL "Waiting ...\n");
		wait_event_interruptible(data->wq, atomic_read(&data->flag) == 1);
		atomic_set(&data->flag, 0);
		break;
	case MY_IOCTL_NOTIFY:
		printk(LOG_LEVEL "Notifying other processes\n");
		atomic_set(&data->flag, 1);
		wake_up_interruptible(&data->wq);
		break;
	default:
		ret = -EINVAL;
	}

	return ret;
}

static const struct file_operations print_fops = {
	.owner = THIS_MODULE,
	.open = print_open,
	.release = print_release,
	.unlocked_ioctl = print_ioctl,
};

static int print_init(void)
{
	int err;
	int i;

	err = register_chrdev_region(MKDEV(MY_MAJOR, MY_MINOR),
			NUM_MINORS, MODULE_NAME);
	if (err != 0) {
		printk(KERN_ERR "register_chrdev_region");
		return err;
	}

	for (i = 0; i < NUM_MINORS; i++) {
		cdev_init(&devs[i].cdev, &print_fops);
		cdev_add(&devs[i].cdev, MKDEV(MY_MAJOR, i), 1);
		atomic_set(&devs[i].flag, 0);
		init_waitqueue_head(&devs[i].wq);
	}

	return 0;
}

static void print_exit(void)
{
	int i;

	for (i = 0; i < NUM_MINORS; i++)
		cdev_del(&devs[i].cdev);

	unregister_chrdev_region(MKDEV(MY_MAJOR, MY_MINOR), NUM_MINORS);
}

module_init(print_init);
module_exit(print_exit);
