#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <fcntl.h>
#include <sys/ioctl.h>
#include "../include/util.h"

#define DEVICE_PATH	"/dev/notify"

/*
 * Print error message and exit.
 */

static void error(const char *message)
{
	perror(message);
	exit(EXIT_FAILURE);
}

/*
 * Print use case.
 */

static void usage(const char *argv0)
{
	printf("Usage: %s <options>\n options:\n"
			"\tw - wait\n"
			"\tn - notify\n", argv0);
	exit(EXIT_FAILURE);
}

/*
 * Sample run:
 *  ./print_test p		; print ioctl message
 */

int main(int argc, char **argv)
{
	int fd;

	if (argc < 2)
		usage(argv[0]);

	if (strlen(argv[1]) != 1)
		usage(argv[0]);

	fd = open(DEVICE_PATH, O_RDONLY);
	if (fd < 0) {
		perror("open");
		exit(EXIT_FAILURE);
	}

	switch (argv[1][0]) {
	case 'w':				/* wait */
		if (ioctl(fd, MY_IOCTL_WAIT, 0) < 0) {
			perror("ioctl");
			exit(EXIT_FAILURE);
		}
		break;
	case 'n':				/* notify */
		if (ioctl(fd, MY_IOCTL_NOTIFY, 0) < 0) {
			perror("ioctl");
			exit(EXIT_FAILURE);
		}
		break;
	default:
		error("Wrong parameter");
	}

	close(fd);

	return 0;
}
