#include <linux/version.h>
#include <linux/init.h>
#include <linux/module.h>
#include <linux/sched.h>
#include <linux/proc_fs.h>
#include <linux/seq_file.h>

MODULE_DESCRIPTION("Access information through proc entry");
MODULE_AUTHOR("Razvan Deaconescu");
MODULE_LICENSE("Dual BSD/GPL");


#define PROC_ENTRY_NAME		"process_pids"


static int pid_list_seq_show(struct seq_file *seq, void *v)
{
	struct task_struct *p;

	for_each_process(p)
		seq_printf(seq, "%d\n", p->pid);

	return 0;
}

static int pid_list_seq_open(struct inode *inode, struct file *file)
{
	return single_open(file, pid_list_seq_show, NULL);
}

static const struct file_operations pid_proc_file_ops = {
	.owner   = THIS_MODULE,
	.open    = pid_list_seq_open,
	.read    = seq_read,
	.llseek  = seq_lseek,
	.release = single_release,
};

static int __init pid_proc_init(void)
{
	proc_create(PROC_ENTRY_NAME, 0, NULL, &pid_proc_file_ops);
	return 0;
}

static void __exit pid_proc_exit(void)
{
	remove_proc_entry(PROC_ENTRY_NAME, NULL);
}

module_init(pid_proc_init);
module_exit(pid_proc_exit);
